extensions [ csv ]

globals [ ;; to declare the global variables
  pl ;; length of the pitch
  pw ;; width of the pitch
  pl-h ;; half length of the pitch
  pw-h ;; half width of the pitch
  scale-ratio ;; the scale ratio

  $me ;; the current auto-agent (players)
  $ball ;; the ball
  team_A_scores
  team_B_scores
  target-approachability? ;; to check if the targeted object is approachable
  xtarget ;; x coordinate of the destination of the target at time t
  ytarget ;; y coordinate of the destination of the targete at time t
  $last-owner ;; last owner of the ball
  $cur-owner ;; curent owner of the ball
  teammates ;; a list of each player's teammates
  opponents ;; a list of each player's opponents
  opp-players ;; a list of opponents, excluding their goalie
  opp-on-box ;; a list of opponents that are currently on players' role-boxes
  p-our-goal ;; a random visible point of ours goal for each player. it can be different in each time and for each player
  p-opp-goal ;; a random visible point of opponent's goal for each player. it can be different in each time and for each player
  dest-target ;; a patch target that is set once a player own the ball
  p-collision ;; a point that players may collide with the ball

;; roles boxes to simulate the spatial constrains on the pitch
  pitch-patches ;; pitch area
  team-A-goal ;; left goal area
  team-B-goal ;; right goal area
  role-box-check ;; a patch-set incloding the curent agent's role-box patches
  target-patch ;; target patch in case of doing something to the ball

  ;; roles-box patches
  ;; in 3-5-2 formation in teams A and B
  a-penalty b-penalty
  a-goalie b-goalie
  a-lCB b-lCB ;; left center back
  a-CB b-CB ;; mid center back
  a-rCB b-rCB ;; right center back
  a-lDM b-lDM ;; left defensive midfielder
  a-rDM b-rDM ;; right defencive midfielder
  a-lWB b-lWB ;; left wing back
  a-rWB b-rWB ;; right wing back
  a-OM b-OM ;; offensive midfelder (Same as in 4-4-2)
  a-lF b-lF ;; left center forward (Same as in 4-4-2)
  a-rF b-rF ;; right center forward (Same as in 4-4-2)
  ;; in 4-4-2 formation in teams A and B
  a-lB b-lB ;; left center back (Same as in 4-4-3)
  a-rB b-rB ;; right center back (Same as in 4-4-3)
  a-lFB b-lFB ;; left full back (Same as in 4-4-3)
  a-rFB b-rFB ;; right full back (Same as in 4-4-3)
  a-CDM b-CDM ;; defensive midfielder
  a-lSM b-lSM ;; left side midfielder
  a-rSM b-rSM ;; right side midfielder
  ;; in 4-4-3 formation in teams A and B
  a-lM b-lM ;; left center midfielder
  a-CM b-CM ;; center midfielder
  a-rM b-rM ;; right center midfielder
  a-CF b-Cf ;; center forward
  a-LW b-LW ;; left winger
  a-RW b-RW ;; right winger

  goalies ;; roles
  defenders ;; roles
  midfielders ;; roles
  strikers ;; roles

  actions ;; to histogram all players choices of actions

]

breed [ amo-gholams amo-gholam ] ;; a special generation of turtles "mash_gholam"
breed [ balls ball ] ;; the ball
breed [ players player ] ;; players (remember that players still are turtles with the same ID, but special type of them)

balls-own [ dis-move energy possessor ]
players-own [ team role role-center energy stumina pace attacking-prowess shooting agility defensive-prowess teamwork p-heading
  ball-possessor fir-opp sec-opp speed-tshd dir-tshd dis-move running oppLCor oppRCor action act-1 act-2 act-3 act-4 act-5 act-6 act-7 exp-data ]

to draw-the-pitch ;; to draw the pitch elements
  set pl-h max-pxcor - 5
  set pw-h max-pycor - 3
  set pl pl-h * 2 ;; full length of the pitch is 210 patches wich is equivalent to 105m, a standard football pitch
  set pw pw-h * 2 ;; full length of the pitch is 120 patches wich is equivalent to 60m, a standard football pitch
  set scale-ratio pl / 105 ;; to draw the pitch based on standard measurements (each patch here is 0.5m * 0.5m)
  set pitch-patches patches with [(pxcor >= (- pl-h)) and (pxcor <= pl-h) and (pycor <= pw-h) and (pycor >= (- pw-h))]
  set team-A-goal patches with [(pxcor < (- pl-h)) and (pxcor > (- pl-h - 3))
    and (pycor < (3.62 * scale-ratio)) and (pycor > (-3.62 * scale-ratio))]
  set team-B-goal patches with  [(pxcor > pl-h) and (pxcor < pl-h + 3)
    and (pycor < (3.62 * scale-ratio)) and (pycor > (-3.62 * scale-ratio))]

  ask pitch-patches [ set pcolor 55 ]
  ask team-A-goal [ set pcolor random 10]
  ask team-B-goal [ set pcolor random 10]

  ;; drawer agent "Mash Gholam"
  create-amo-gholams 1
  ask amo-gholam 0
  [
    set shape "person"
    set hidden? true
    set color white
    pu

    setxy pl-h pw-h ;; draw bourders
    set heading 180
    pd
    set pen-size 3
    fd pw rt 90 fd pl rt 90 fd pw rt 90 fd pl
    setxy 0 pw-h
    rt 90 fd pw
    pu

    set pen-size 2
    setxy 0 scale-ratio * 9.15 ;; draw the midle circle
    set heading 90
    pd
    repeat 120
    [
      fd 1
      right 3.1
    ]
    pu

    setxy (pl-h - (scale-ratio * 16.5)) scale-ratio * 6.5 ;; draw the right penalty circle
    set heading 225
    pd
    repeat 29
    [
      fd 1
      left 3.15
    ]
    pu

    setxy (- pl-h + (scale-ratio * 16.5)) scale-ratio * 6.5 ;; draw the left penalty circle
    set heading 135
    pd
    repeat 29
    [
      fd 1
      right 3.15
    ]
    pu

    setxy pl-h (scale-ratio * 20.12) ;; draw the right 18 yards area
    set heading 270
    pd
    fd scale-ratio * 16.5 lt 90 fd scale-ratio * 40.24 lt 90 fd scale-ratio * 16.5
    pu

    setxy (- pl-h) (scale-ratio * 20.12) ;; draw the left 18 yards area
    set heading 90
    pd
    fd scale-ratio * 16.5 rt 90 fd scale-ratio * 40.24 rt 90 fd scale-ratio * 16.5
    pu

    setxy pl-h scale-ratio * 9.12 ;; draw the right 6 yards area
    set heading 270
    pd
    fd scale-ratio * 5.5 lt 90 fd scale-ratio * 18.32 lt 90 fd scale-ratio * 5.5
    pu

    setxy (- pl-h) scale-ratio * 9.12 ;; draw the left 6 yards area
    set heading 90
    pd
    fd scale-ratio * 5.5 rt 90 fd scale-ratio * 18.32 rt 90 fd scale-ratio * 5.5
    pu

    setxy 0 0 ;; draw the midle point
    set heading 0
    pd
    set pen-size 3
    fd 0.2 rt 90 fd 0.2 rt 90 fd 0.4 rt 90 fd 0.4 rt 90 fd 0.4
    pu

    setxy (pl-h - (scale-ratio * 11)) 0 ;; draw the right penalty point
    set heading 0
    pd
    fd 0.2 rt 90 fd 0.2 rt 90 fd 0.4 rt 90 fd 0.4 rt 90 fd 0.4
    pu

    setxy (- pl-h + (scale-ratio * 11)) 0 ;; draw the left penalty point
    set heading 0
    pd
    fd 0.2 rt 90 fd 0.2 rt 90 fd 0.4 rt 90 fd 0.4 rt 90 fd 0.4
    pu

    setxy pl-h scale-ratio * 3.62 ;; draw the right goal
    set heading 90
    set pen-size 3
    pd
    fd scale-ratio * 1.22 rt 90 fd scale-ratio * 7.32 rt 90 fd scale-ratio * 1.22
    pu

    setxy (- pl-h) scale-ratio * 3.62 ;; draw the left goal
    set heading 270
    pd
    fd scale-ratio * 1.22 lt 90 fd scale-ratio * 7.32 lt 90 fd scale-ratio * 1.22
    pu

    die
  ]
end

to create-autoAgents ;; to create players and allocate their initial attributes.
  ;; players will have some indigenous charactrestics which influence their movement decisions.
     ;; These attributes is considered as zero-order factors in movement analysis.
  create-players 22 [
    set size 4
    set label who
    facexy 0 0
    set exp-data []
  ]

  ask players with [ who > 0 and who <= 11 ] [ set team "team-A" set color blue set energy 100 set heading 90 set oppLCor patch 105 -60 set oppRCor patch 105 60 ] ;; FC Barcelona
  ask players with [ who > 11 and who <= 22 ] [ set team "team-B" set color red set energy 100 set heading 270 set oppLCor patch -105 60 set oppRCor patch -105 -60  ] ;; Real Madrid

  ;; All of these indigenous attributes have been coppied from FC Barcelona (Team-A) and Real Madrid (Team-B) in FIFA 18 video game
     ;; (see https://www.futhead.com/18/clubs/).
  ask player 1 [ set stumina 90 set pace 46 set attacking-prowess 5 set shooting 95 set agility 10 set defensive-prowess 100 set teamwork 95 ] ;; Stegen
  ask player 2 [ set stumina 79 set pace 74 set attacking-prowess 67 set shooting 64 set agility 71 set defensive-prowess 82 set teamwork 71 ] ;; Umtiti
  ask player 3 [ set stumina 76 set pace 65 set attacking-prowess 60 set shooting 61 set agility 66 set defensive-prowess 86 set teamwork 69 ] ;; Pique
  ask player 4 [ set stumina 61 set pace 94 set attacking-prowess 74 set shooting 72 set agility 84 set defensive-prowess 85 set teamwork 78 ] ;; Alba
  ask player 5 [ set stumina 78 set pace 80 set attacking-prowess 72 set shooting 69 set agility 80 set defensive-prowess 86 set teamwork 85 ] ;; Roberto
  ask player 6 [ set stumina 81 set pace 40 set attacking-prowess 68 set shooting 62 set agility 78 set defensive-prowess 82 set teamwork 79 ] ;; Busquets
  ask player 7 [ set stumina 58 set pace 71 set attacking-prowess 76 set shooting 70 set agility 90 set defensive-prowess 58 set teamwork 87 ] ;; Iniesta
  ask player 8 [ set stumina 66 set pace 65 set attacking-prowess 83 set shooting 84 set agility 82 set defensive-prowess 62 set teamwork 86 ] ;; Rakitic
  ask player 9 [ set stumina 84 set pace 84 set attacking-prowess 92 set shooting 94 set agility 90 set defensive-prowess 50 set teamwork 83 ] ;; Suarez
  ask player 10 [ set stumina 69 set pace 93 set attacking-prowess 94 set shooting 94 set agility 99 set defensive-prowess 37 set teamwork 91 ] ;; Messi
  ask player 11 [ set stumina 65 set pace 93 set attacking-prowess 76 set shooting 73 set agility 88 set defensive-prowess 33 set teamwork 77 ] ;; Deulofeu

  ask player 12 [ set stumina 88 set pace 55 set attacking-prowess 5 set shooting 95 set agility 10 set defensive-prowess 100 set teamwork 95 ] ;; Navas
  ask player 13 [ set stumina 94 set pace 83 set attacking-prowess 75 set shooting 75 set agility 85 set defensive-prowess 96 set teamwork 85 ] ;; Ramos
  ask player 14 [ set stumina 79 set pace 79 set attacking-prowess 50 set shooting 45 set agility 65 set defensive-prowess 84 set teamwork 61 ] ;; Varane
  ask player 15 [ set stumina 79 set pace 78 set attacking-prowess 75 set shooting 69 set agility 85 set defensive-prowess 83 set teamwork 80 ] ;; Marcelo
  ask player 16 [ set stumina 78 set pace 82 set attacking-prowess 56 set shooting 45 set agility 79 set defensive-prowess 83 set teamwork 73 ] ;; Carvajal
  ask player 17 [ set stumina 72 set pace 52 set attacking-prowess 77 set shooting 86 set agility 85 set defensive-prowess 87 set teamwork 92 ] ;; Kroos
  ask player 18 [ set stumina 68 set pace 75 set attacking-prowess 83 set shooting 81 set agility 91 set defensive-prowess 76 set teamwork 89 ] ;; Modric
  ask player 19 [ set stumina 66 set pace 73 set attacking-prowess 88 set shooting 88 set agility 91 set defensive-prowess 59 set teamwork 87 ] ;; Isco
  ask player 20 [ set stumina 84 set pace 92 set attacking-prowess 95 set shooting 96 set agility 93 set defensive-prowess 40 set teamwork 86 ] ;; Ronaldo
  ask player 21 [ set stumina 83 set pace 97 set attacking-prowess 93 set shooting 93 set agility 91 set defensive-prowess 63 set teamwork 89 ] ;; Bale
  ask player 22 [ set stumina 76 set pace 79 set attacking-prowess 84 set shooting 86 set agility 84 set defensive-prowess 29 set teamwork 81 ] ;; Benzema
end

to create-contextual-objects ;; to create a role-box for each player in the pitch and the ball
;; These boxes and the ball are considered as contextual objects
  create-balls 1 [ ;; to create the ball object
    set color black
    set size 2.5
    set shape "ball"
    set $ball self
    set $cur-owner nobody
    set $last-owner nobody
  ]

  ;; Role boxes in 3-5-2 formation
     ;; Note; some of the roles are almost the same in different formations. For example, right center forward
        ;; (rCF) and left center forwared (lCF), or goalie.
  set a-penalty patches with [ pxcor >= -105 and pxcor <= -72 and pycor >= -40 and pycor <= 40 ] ;; penalty box
  set a-goalie patches with [ pxcor >= -105 and pxcor <= -95 and pycor >= -18 and pycor <= 18 ]
  set b-penalty patches with [ pxcor >= 72 and pxcor <= 105 and pycor >= -40 and pycor <= 40 ] ;; penalty box
  set b-goalie patches with [ pxcor >= 95 and pxcor <= 105 and pycor >= -18 and pycor <= 18 ]
  set a-lCB patches with [ pxcor >= -105 and pxcor <= -40 and pycor >= 0 and pycor <= 60 ]
  set b-lCB patches with [ pxcor >= 40 and pxcor <= 105 and pycor >= -60 and pycor <= -0 ]
  set a-CB patches with [ pxcor >= -105 and pxcor <= -40 and pycor >= -40 and pycor <= 40 ]
  set b-CB patches with [ pxcor >= 40 and pxcor <= 105 and pycor >= -40 and pycor <= 40 ]
  set a-rCB patches with [ pxcor >= -105 and pxcor <= -40 and pycor >= -60 and pycor <= -0 ]
  set b-rCB patches with [ pxcor >= 40 and pxcor <= 105 and pycor >= 0 and pycor <= 60 ]
  set a-lDM patches with [ pxcor >= -72 and pxcor <= 8 and pycor >= -4 and pycor <= 60 ]
  set b-lDM patches with [ pxcor >= -8 and pxcor <= 72 and pycor >= -60 and pycor <= 4 ]
  set a-rDM patches with [ pxcor >= -72 and pxcor <= 8 and pycor >= -60 and pycor <= 4 ]
  set b-rDM patches with [ pxcor >= -8 and pxcor <= 72 and pycor >= -4 and pycor <= 60 ]
  set a-lWB patches with [ pxcor >= -20 and pxcor <= 105 and pycor >= 20 and pycor <= 60 ]
  set b-lWB patches with [ pxcor >= -105 and pxcor <= 20 and pycor >= -60 and pycor <= -20 ]
  set a-rWB patches with [ pxcor >= -20 and pxcor <= 105 and pycor >= -60 and pycor <= -20 ]
  set b-rWB patches with [ pxcor >= -105 and pxcor <= 20 and pycor >= 20 and pycor <= 60 ]
  set a-OM patches with [ pxcor >= -20 and pxcor <= 60 and pycor >= -40 and pycor <= 40 ]
  set b-OM patches with [ pxcor >= -60 and pxcor <= 20 and pycor >= -40 and pycor <= 40 ]
  set a-lF patches with [ pxcor >= 30 and pxcor <= 105 and pycor >= -4 and pycor <= 60 ]
  set b-lF patches with [ pxcor >= -105 and pxcor <= -30 and pycor >= -60 and pycor <= 4 ]
  set a-rF patches with [ pxcor >= 30 and pxcor <= 105 and pycor >= -60 and pycor <= 4 ]
  set b-rF patches with [ pxcor >= -105 and pxcor <= -30 and pycor >= -4 and pycor <= 60 ]

  ;; Role boxes in 4-4-2 formation
  set a-lB patches with [ pxcor >= -105 and pxcor <= -34 and pycor >= -8 and pycor <= 60 ]
  set b-lB patches with [ pxcor >= 34 and pxcor <= 105 and pycor >= -60 and pycor <= 8 ]
  set a-rB patches with [ pxcor >= -105 and pxcor <= -34 and pycor >= -60 and pycor <= 8 ]
  set b-rB patches with [ pxcor >= 34 and pxcor <= 105 and pycor >= -8 and pycor <= 60 ]
  set a-lFB patches with [ pxcor >= -105 and pxcor <= 5 and pycor >= 18 and pycor <= 60 ]
  set b-lFB patches with [ pxcor >= -5 and pxcor <= 105 and pycor >= -60 and pycor <= -18 ]
  set a-rFB patches with [ pxcor >= -105 and pxcor <= 5 and pycor >= -60 and pycor <= -18 ]
  set b-rFB patches with [ pxcor >= -5 and pxcor <= 105 and pycor >= 18 and pycor <= 60 ]
  set a-CDM patches with [ pxcor >= -80 and pxcor <= 10 and pycor >= -35 and pycor <= 35 ]
  set b-CDM patches with [ pxcor >= -10 and pxcor <= 80 and pycor >= -35 and pycor <= 35 ]
  set a-lSM patches with [ pxcor >= -15 and pxcor <= 105 and pycor >= 20 and pycor <= 60 ]
  set b-lSM patches with [ pxcor >= -105 and pxcor <= 15 and pycor >= -60 and pycor <= -20 ]
  set a-rSM patches with [ pxcor >= -15 and pxcor <= 105 and pycor >= -60 and pycor <= -20 ]
  set b-rSM patches with [ pxcor >= -105 and pxcor <= 15 and pycor >= 20 and pycor <= 60 ]

  ;; Role boxes in 4-3-3 formation
  set a-lM patches with [ pxcor >= -44 and pxcor <= 52 and pycor >= 6 and pycor <= 60 ]
  set b-lM patches with [ pxcor >= -52 and pxcor <= 44 and pycor >= -60 and pycor <= -6 ]
  set a-CM patches with [ pxcor >= -60 and pxcor <= 40 and pycor >= -32 and pycor <= 32 ]
  set b-CM patches with [ pxcor >= -40 and pxcor <= 60 and pycor >= -32 and pycor <= 32 ]
  set a-rM patches with [ pxcor >= -44 and pxcor <= 52 and pycor >= -60 and pycor <= -6 ]
  set b-rM patches with [ pxcor >= -52 and pxcor <= 44 and pycor >= 6 and pycor <= 60 ]
  set a-CF patches with [ pxcor >= 40 and pxcor <= 105 and pycor >= -35 and pycor <= 35 ]
  set b-CF patches with [ pxcor >= -105 and pxcor <= -40 and pycor >= -35 and pycor <= 35 ]
  set a-lW patches with [ pxcor >= 20 and pxcor <= 105 and pycor >= 8 and pycor <= 60 ]
  set b-lW patches with [ pxcor >= -105 and pxcor <= -20 and pycor >= -60 and pycor <= -8 ]
  set a-rW patches with [ pxcor >= 20 and pxcor <= 105 and pycor >= -60 and pycor <= -8 ]
  set b-rW patches with [ pxcor >= -105 and pxcor <= -20 and pycor >= 8 and pycor <= 60 ]
end

to show-role-boxes ;; to highlight the selected rol boxes
  ask pitch-patches [ set pcolor 55 ]

  if (a-role-box != "none") [ ask (run-result (word "a-" a-role-box)) [ set pcolor orange ] ]

  if (b-role-box != "none") [ ask (run-result (word "b-" b-role-box)) [ set pcolor yellow ] ]
end

To role-positioning ;; to initiall set players' positions based on their roles in the formation

  ask $ball [ setxy 0 0]

  ask players [ pu ]
  if (A-formation = "4-4-2") [
    ask player 1 [ setxy (mean [pxcor] of a-goalie ) (mean [pycor] of a-goalie ) set role "a-goalie"  ]
    ask player 2 [ setxy (mean [pxcor] of a-lB ) (mean [pycor] of a-lB ) set role "a-lB" ]
    ask player 3 [ setxy (mean [pxcor] of a-rB) (mean [pycor] of a-rB) set role "a-rB" ]
    ask player 4 [ setxy (mean [pxcor] of a-lFB) (mean [pycor] of a-lFB) set role "a-lFB" ]
    ask player 5 [ setxy (mean [pxcor] of a-rFB) (mean [pycor] of a-rFB) set role "a-rFB" ]
    ask player 6 [ setxy (mean [pxcor] of a-CDM) (mean [pycor] of a-CDM) set role "a-CDM" ]
    ask player 7 [ setxy (mean [pxcor] of a-lSM) (mean [pycor] of a-lSM) set role "a-lSM" ]
    ask player 8 [ setxy (mean [pxcor] of a-rSM) (mean [pycor] of a-rSM) set role "a-rSM" ]
    ask player 10 [ setxy (mean [pxcor] of a-OM) (mean [pycor] of a-OM) set role "a-OM" ]
    ask player 9 [ setxy (mean [pxcor] of a-lF) (mean [pycor] of a-lF) set role "a-lF" ]
    ask player 11 [ setxy (mean [pxcor] of a-rF) (mean [pycor] of a-rF) set role "a-rF" ]
  ]

  if (B-formation = "4-4-2") [
    ask player 12 [ setxy (mean [pxcor] of b-goalie) (mean [pycor] of b-goalie) set role "b-goalie" ]
    ask player 13 [ setxy (mean [pxcor] of b-lB) (mean [pycor] of b-lB) set role "b-lB" ]
    ask player 14 [ setxy (mean [pxcor] of b-rB) (mean [pycor] of b-rB) set role "b-rB" ]
    ask player 15 [ setxy (mean [pxcor] of b-lFB) (mean [pycor] of b-lFB) set role "b-lFB" ]
    ask player 16 [ setxy (mean [pxcor] of b-rFB) (mean [pycor] of b-rFB) set role "b-rFB" ]
    ask player 17 [ setxy (mean [pxcor] of b-CDM) (mean [pycor] of b-CDM) set role "b-CDM" ]
    ask player 18 [ setxy (mean [pxcor] of b-lSM) (mean [pycor] of b-lSM) set role "b-lSM" ]
    ask player 19 [ setxy (mean [pxcor] of b-rSM) (mean [pycor] of b-rSM) set role "b-rSM" ]
    ask player 20 [ setxy (mean [pxcor] of b-OM) (mean [pycor] of b-OM) set role "b-OM" ]
    ask player 21 [ setxy (mean [pxcor] of b-lF) (mean [pycor] of b-lF) set role "b-lF" ]
    ask player 22 [ setxy (mean [pxcor] of b-rF) (mean [pycor] of b-rF) set role "b-rF" ]
  ]

  if (A-formation = "4-3-3") [
    ask player 1 [ setxy (mean [pxcor] of a-goalie) (mean [pycor] of a-goalie) set role "a-goalie" ]
    ask player 2 [ setxy (mean [pxcor] of a-lB) (mean [pycor] of a-lB) set role "a-lB" ]
    ask player 3 [ setxy (mean [pxcor] of a-rB) (mean [pycor] of a-rB) set role "a-rB" ]
    ask player 4 [ setxy (mean [pxcor] of a-lFB) (mean [pycor] of a-lFB) set role "a-lFB" ]
    ask player 5 [ setxy (mean [pxcor] of a-rFB) (mean [pycor] of a-rFB) set role "a-rFB" ]
    ask player 6 [ setxy (mean [pxcor] of a-CM) (mean [pycor] of a-CM) set role "a-CM" ]
    ask player 7 [ setxy (mean [pxcor] of a-lM) (mean [pycor] of a-lM) set role "a-lM" ]
    ask player 8 [ setxy (mean [pxcor] of a-rM) (mean [pycor] of a-rM) set role "a-rM" ]
    ask player 9 [ setxy (mean [pxcor] of a-CF) (mean [pycor] of a-CF) set role "a-CF" ]
    ask player 11 [ setxy (mean [pxcor] of a-lW) (mean [pycor] of a-lW) set role "a-lW" ]
    ask player 10 [ setxy (mean [pxcor] of a-rW) (mean [pycor] of a-rW) set role "a-rW" ]
  ]

  if (B-formation = "4-3-3") [
    ask player 12 [ setxy (mean [pxcor] of b-goalie) (mean [pycor] of b-goalie) set role "b-goalie" ]
    ask player 13 [ setxy (mean [pxcor] of b-lB) (mean [pycor] of b-lB) set role "b-lB" ]
    ask player 14 [ setxy (mean [pxcor] of b-rB) (mean [pycor] of b-rB) set role "b-rB" ]
    ask player 15 [ setxy (mean [pxcor] of b-lFB) (mean [pycor] of b-lFB) set role "b-lFB" ]
    ask player 16 [ setxy (mean [pxcor] of b-rFB) (mean [pycor] of b-rFB) set role "b-rFB" ]
    ask player 17 [ setxy (mean [pxcor] of b-CM) (mean [pycor] of b-CM) set role "b-CM" ]
    ask player 18 [ setxy (mean [pxcor] of b-lM) (mean [pycor] of b-lM) set role "b-lM" ]
    ask player 19 [ setxy (mean [pxcor] of b-rM) (mean [pycor] of b-rM) set role "b-rM" ]
    ask player 20 [ setxy (mean [pxcor] of b-CF) (mean [pycor] of b-CF) set role "b-CF" ]
    ask player 21 [ setxy (mean [pxcor] of b-lW) (mean [pycor] of b-lW) set role "b-lW" ]
    ask player 22 [ setxy (mean [pxcor] of b-rW) (mean [pycor] of b-rW) set role "b-rW" ]
  ]

  if (A-formation = "3-5-2") [
    ask player 1 [ setxy (mean [pxcor] of a-goalie) (mean [pycor] of a-goalie) set role "a-goalie" ]
    ask player 2 [ setxy (mean [pxcor] of a-CB) (mean [pycor] of a-CB) set role "a-CB" ]
    ask player 3 [ setxy (mean [pxcor] of a-lCB) (mean [pycor] of a-lCB) set role "a-lCB" ]
    ask player 4 [ setxy (mean [pxcor] of a-rCB) (mean [pycor] of a-rCB) set role "a-rCB" ]
    ask player 6 [ setxy (mean [pxcor] of a-lDM) (mean [pycor] of a-lDM) set role "a-lDM" ]
    ask player 5 [ setxy (mean [pxcor] of a-rDM) (mean [pycor] of a-rDM) set role "a-rDM" ]
    ask player 7 [ setxy (mean [pxcor] of a-lWB) (mean [pycor] of a-lWB) set role "a-lWB" ]
    ask player 8 [ setxy (mean [pxcor] of a-rWB) (mean [pycor] of a-rWB) set role "a-rWB" ]
    ask player 10 [ setxy (mean [pxcor] of a-OM) (mean [pycor] of a-OM) set role "a-OM" ]
    ask player 9 [ setxy (mean [pxcor] of a-lF) (mean [pycor] of a-lF) set role "a-lF" ]
    ask player 11 [ setxy (mean [pxcor] of a-rF) (mean [pycor] of a-rF) set role "a-rF" ]
  ]
  if (B-formation = "3-5-2") [
    ask player 12 [ setxy (mean [pxcor] of b-goalie) (mean [pycor] of b-goalie) set role "b-goalie" ]
    ask player 14 [ setxy (mean [pxcor] of b-CB) (mean [pycor] of b-CB) set role "b-CB" ]
    ask player 15 [ setxy (mean [pxcor] of b-lCB) (mean [pycor] of b-lCB) set role "b-lCB" ]
    ask player 16 [ setxy (mean [pxcor] of b-rCB) (mean [pycor] of b-rCB) set role "b-rCB" ]
    ask player 17 [ setxy (mean [pxcor] of b-lDM) (mean [pycor] of b-lDM) set role "b-lDM" ]
    ask player 13 [ setxy (mean [pxcor] of b-rDM) (mean [pycor] of b-rDM) set role "b-rDM" ]
    ask player 18 [ setxy (mean [pxcor] of b-lWB) (mean [pycor] of b-lWB) set role "b-lWB" ]
    ask player 19 [ setxy (mean [pxcor] of b-rWB) (mean [pycor] of b-rWB) set role "b-rWB" ]
    ask player 20 [ setxy (mean [pxcor] of b-OM) (mean [pycor] of b-OM) set role "b-OM" ]
    ask player 21 [ setxy (mean [pxcor] of b-lF) (mean [pycor] of b-lF) set role "b-lF" ]
    ask player 22 [ setxy (mean [pxcor] of b-rF) (mean [pycor] of b-rF) set role "b-rF" ]
  ]

  ask players [ set role-center patch (mean [pxcor] of (run-result role)) (mean [pycor] of (run-result role)) ] ;; pd ]
end

to assign-opponent ;; to mark one or two opponent for each player

  if (A-formation = "3-5-2") and (B-formation = "3-5-2") [
    ask player 2 [ set fir-opp 21 set sec-opp 22 ]
    ask player 3 [ set fir-opp 22 set sec-opp 19 ]
    ask player 4 [ set fir-opp 21 set sec-opp 18 ]
    ask player 5 [ set fir-opp 18 set sec-opp 20 ]
    ask player 6 [ set fir-opp 19 set sec-opp 20 ]
    ask player 7 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 13 set sec-opp 17 ]
    ask player 11 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 7 set sec-opp 10 ]
    ask player 14 [ set fir-opp 9 set sec-opp 11 ]
    ask player 15 [ set fir-opp 11 set sec-opp 8 ]
    ask player 16 [ set fir-opp 9 set sec-opp 7 ]
    ask player 17 [ set fir-opp 8 set sec-opp 10 ]
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 5 set sec-opp 6 ]
    ask player 21 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "3-5-2") and (B-formation = "4-4-2") [
    ask player 2 [ set fir-opp 21 set sec-opp 22 ]
    ask player 3 [ set fir-opp 22 set sec-opp 19 ]
    ask player 4 [ set fir-opp 21 set sec-opp 18 ]
    ask player 5 [ set fir-opp 18 set sec-opp 20 ]
    ask player 6 [ set fir-opp 19 set sec-opp 20 ]
    ask player 7 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 14 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 11 set sec-opp 8 ]
    ask player 14 [ set fir-opp 9 set sec-opp 7 ]
    ask player 15 [ set fir-opp 8 set sec-opp 11 ]
    ask player 16 [ set fir-opp 7 set sec-opp 9 ]
    ask player 17 [ set fir-opp 10 set sec-opp "Null" ] ;; only one marked opponent
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 5 set sec-opp 6 ]
    ask player 21 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "3-5-2") and (B-formation = "4-3-3") [
    ask player 2 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 3 [ set fir-opp 22 set sec-opp 20 ]
    ask player 4 [ set fir-opp 21 set sec-opp 20 ]
    ask player 5 [ set fir-opp 21 set sec-opp 18 ]
    ask player 6 [ set fir-opp 22 set sec-opp 19 ]
    ask player 7 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 14 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 11 set sec-opp 8 ]
    ask player 14 [ set fir-opp 9 set sec-opp 7 ]
    ask player 15 [ set fir-opp 8 set sec-opp 11 ]
    ask player 16 [ set fir-opp 7 set sec-opp 9 ]
    ask player 17 [ set fir-opp 10 set sec-opp "Null" ] ;; only one marked opponent
    ask player 18 [ set fir-opp 5 set sec-opp 10 ]
    ask player 19 [ set fir-opp 6 set sec-opp 10 ]
    ask player 20 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
    ask player 21 [ set fir-opp 4 set sec-opp "Null" ]
    ask player 22 [ set fir-opp 3 set sec-opp "Null" ]
  ]

  if (A-formation = "4-4-2") and (B-formation = "4-4-2") [
    ask player 2 [ set fir-opp 22 set sec-opp 19 ]
    ask player 3 [ set fir-opp 21 set sec-opp 18 ]
    ask player 4 [ set fir-opp 19 set sec-opp 22 ]
    ask player 5 [ set fir-opp 18 set sec-opp 21 ]
    ask player 6 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 7 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 14 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 11 set sec-opp 8 ]
    ask player 14 [ set fir-opp 9 set sec-opp 7 ]
    ask player 15 [ set fir-opp 8 set sec-opp 11 ]
    ask player 16 [ set fir-opp 7 set sec-opp 9 ]
    ask player 17 [ set fir-opp 10 set sec-opp "Null" ] ;; only one marked opponent
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "4-4-2") and (B-formation = "3-5-2") [
    ask player 2 [ set fir-opp 22 set sec-opp 19 ]
    ask player 3 [ set fir-opp 21 set sec-opp 18 ]
    ask player 4 [ set fir-opp 19 set sec-opp 22 ]
    ask player 5 [ set fir-opp 18 set sec-opp 21 ]
    ask player 6 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 7 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 13 set sec-opp 17 ]
    ask player 11 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 7 set sec-opp 10 ]
    ask player 14 [ set fir-opp 9 set sec-opp 11 ]
    ask player 15 [ set fir-opp 11 set sec-opp 8 ]
    ask player 16 [ set fir-opp 9 set sec-opp 7 ]
    ask player 17 [ set fir-opp 8 set sec-opp 10 ]
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "4-4-2") and (B-formation = "4-3-3") [
    ask player 2 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 3 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 4 [ set fir-opp 21 set sec-opp "Null" ] ;; only one marked opponent
    ask player 5 [ set fir-opp 22 set sec-opp "Null" ] ;; only one marked opponent
    ask player 6 [ set fir-opp 18 set sec-opp 19 ]
    ask player 7 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 8 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 9 [ set fir-opp 14 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 17 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 13 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 11 set sec-opp 8 ]
    ask player 14 [ set fir-opp 9 set sec-opp 7 ]
    ask player 15 [ set fir-opp 8 set sec-opp 11 ]
    ask player 16 [ set fir-opp 7 set sec-opp 9 ]
    ask player 17 [ set fir-opp 10 set sec-opp "Null" ] ;; only one marked opponent
    ask player 18 [ set fir-opp 5 set sec-opp 6 ]
    ask player 19 [ set fir-opp 6 set sec-opp 4 ]
    ask player 20 [ set fir-opp 2 set sec-opp 3 ]
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "4-3-3") and (B-formation = "4-3-3") [
    ask player 2 [ set fir-opp 20 set sec-opp 22  ]
    ask player 3 [ set fir-opp 20 set sec-opp 21 ]
    ask player 4 [ set fir-opp 22 set sec-opp 19 ]
    ask player 5 [ set fir-opp 21 set sec-opp 18 ]
    ask player 6 [ set fir-opp 18 set sec-opp 19 ]
    ask player 7 [ set fir-opp 19 set sec-opp 17 ]
    ask player 8 [ set fir-opp 18 set sec-opp 17 ]
    ask player 9 [ set fir-opp 13 set sec-opp 14 ]
    ask player 10 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 9 set sec-opp 10 ]
    ask player 14 [ set fir-opp 9 set sec-opp 11 ]
    ask player 15 [ set fir-opp 10 set sec-opp 8 ]
    ask player 16 [ set fir-opp 11 set sec-opp 7 ]
    ask player 17 [ set fir-opp 7 set sec-opp 8 ]
    ask player 18 [ set fir-opp 6 set sec-opp 5 ]
    ask player 19 [ set fir-opp 6 set sec-opp 4 ]
    ask player 20 [ set fir-opp 2 set sec-opp 3 ]
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "4-3-3") and (B-formation = "3-5-2") [
    ask player 2 [ set fir-opp 22 set sec-opp 19 ]
    ask player 3 [ set fir-opp 21 set sec-opp 18 ]
    ask player 4 [ set fir-opp 19 set sec-opp 20 ]
    ask player 5 [ set fir-opp 18 set sec-opp 21 ]
    ask player 6 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 7 [ set fir-opp 13 set sec-opp 20 ]
    ask player 8 [ set fir-opp 17 set sec-opp 20 ]
    ask player 9 [ set fir-opp 14 set sec-opp "Null" ] ;; only one marked opponent
    ask player 10 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 11 set sec-opp 7 ]
    ask player 14 [ set fir-opp 9 set sec-opp "Null" ] ;; only one marked opponent
    ask player 15 [ set fir-opp 10 set sec-opp 9 ]
    ask player 16 [ set fir-opp 11 set sec-opp 9 ]
    ask player 17 [ set fir-opp 10 set sec-opp 8 ]
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]

  if (A-formation = "4-3-3") and (B-formation = "4-4-2") [
    ask player 2 [ set fir-opp 22 set sec-opp 19 ]
    ask player 3 [ set fir-opp 21 set sec-opp 18 ]
    ask player 4 [ set fir-opp 19 set sec-opp 22 ]
    ask player 5 [ set fir-opp 18 set sec-opp 21 ]
    ask player 6 [ set fir-opp 20 set sec-opp "Null" ] ;; only one marked opponent
    ask player 7 [ set fir-opp 13 set sec-opp 17 ]
    ask player 8 [ set fir-opp 17 set sec-opp 17 ]
    ask player 9 [ set fir-opp 14 set sec-opp 14 ]
    ask player 10 [ set fir-opp 15 set sec-opp "Null" ] ;; only one marked opponent
    ask player 11 [ set fir-opp 16 set sec-opp "Null" ] ;; only one marked opponent
    ask player 13 [ set fir-opp 9 set sec-opp 10 ]
    ask player 14 [ set fir-opp 9 set sec-opp 11 ]
    ask player 15 [ set fir-opp 10 set sec-opp 8 ]
    ask player 16 [ set fir-opp 11 set sec-opp 7 ]
    ask player 17 [ set fir-opp 7 set sec-opp 8]
    ask player 18 [ set fir-opp 5 set sec-opp "Null" ] ;; only one marked opponent
    ask player 19 [ set fir-opp 4 set sec-opp "Null" ] ;; only one marked opponent
    ask player 20 [ set fir-opp 6 set sec-opp "Null" ] ;; only one marked opponent
    ask player 21 [ set fir-opp 3 set sec-opp "Null" ] ;; only one marked opponent
    ask player 22 [ set fir-opp 2 set sec-opp "Null" ] ;; only one marked opponent
  ]
end

to setup
  ca ;; Clear all

  draw-the-pitch
  create-autoAgents
  create-contextual-objects
  role-positioning

  set goalies ["a-goalie" "b-goalie"]
  set defenders ["a-lCB" "a-CB" "a-rCB" "a-lB" "a-rB" "a-lFB" "a-rFB" "b-lCB" "b-CB" "b-rCB" "b-lB" "b-rB" "b-lFB" "b-rFB"]
  set midfielders ["a-lDM" "a-rDM" "a-lWB" "a-rWB" "a-OM" "a-CDM" "a-lSM" "a-rSM" "a-lM" "a-CM" "a-rM"
                   "b-lDM" "b-rDM" "b-lWB" "b-rWB" "b-OM" "b-CDM" "b-lSM" "b-rSM" "b-lM" "b-CM" "b-rM"]
  set strikers ["a-lF" "a-rF" "a-lW" "a-CF" "a-rW" "b-lF" "b-rF" "b-lW" "b-CF" "b-rW"]

  assign-opponent

  create-turtles 3 ;; these agents are used to calculate the movement limitations of the ball and players in each tick

  stop-inspecting-dead-agents
  reset-ticks
end

to calculate-energy ;; to control agents' endogenous states,
  ;; 100 level energy is dedicated to each player that is reduced over passing time.

  ;; the energy concumption rate is assumed to be 1 percent in each 500 time intervals.

  let max-running ((stumina / 100) * 200)
  set energy (100 - (ticks / (stumina * 5)))
end

to check-zero-order ;; to simulate players' endogenous abilities (zero-order)
  ;; first step is to calculate players' maximum velocity (speed and turnning threshold) in each tick.

  ;; pace and agility are initiated and decreased over time based on the consumed energy that itself
     ;; is a function time.

  ;; travel distance (speed) threshold for each player in each tick is a function of its current state
     ;; of energy multiple by its initial pace (2 patch, 1 meter, per tick, 100 milliseconds).

  calculate-energy
  set speed-tshd ((pace / 100) * (energy / 100) * 2) ;; current speed threshold

  ;; turnning threshold (in minus and pluse range to cover the 180 degree) is a function of players' agility
     ;; and their current state of energy
  set dir-tshd ((agility / 100) * (energy / 100) * 60)

  ;; to put agent 25 and 26 in two maximume dicplacement position for the current palyer at left and right.
  ask turtle 25 [
    set heading ([heading] of $me - [dir-tshd] of $me)
    let x-end1 ([xcor] of $me + (dx * [speed-tshd] of $me))
    let y-end1 ([ycor] of $me + (dy * [speed-tshd] of $me))
    setxy x-end1 y-end1
  ]
  ask turtle 26 [
    setxy [xcor] of $me [ycor] of $me
    set heading ([heading] of $me + [dir-tshd] of $me)
    let x-end2 ([xcor] of $me + (dx * [speed-tshd] of $me))
    let y-end2 ([ycor] of $me + (dy * [speed-tshd] of $me))
    setxy x-end2 y-end2
  ]
end

to check-first-order ;; to create spatial constrains in the model
  set role-box-check (run-result role) ;; make players aware of its role-box area

  let x-opp-goal 0
  ifelse (team = "team-A") [ ;; to make players aware of their and the opponent goal area
    set p-our-goal max-one-of team-A-goal [pxcor]
    set x-opp-goal min-one-of team-B-goal [pxcor]
    set p-opp-goal one-of (patches with [(pxcor = [pxcor] of x-opp-goal) and (pycor > -15 and pycor < 15)])
  ] [
    set p-our-goal min-one-of team-B-goal [pxcor]
    set x-opp-goal max-one-of team-A-goal [pxcor]
    set p-opp-goal one-of (patches with [(pxcor = [pxcor] of x-opp-goal) and (pycor > -15 and pycor < 15)])
  ]
end

to check-second-order ;; to make players aware of their teammates and the opponent team
  let my-team team

  set opponents players with [team != my-team]
  let opp-goalie opponents with-min [who]
  ask opp-goalie [ set opp-players other opponents ] ;; excluding the goalie from the opponents agentset
  set opp-on-box opp-players with [member? patch-here role-box-check]

  set teammates other players with [team = my-team]
;  set teammates teammates with [self != myself]
end

to check-approachability ;; to check if the target object is approachable by the current player
                            ;; this procedure is a combination of zero- and first-order factors.
  set target-approachability? false
  set p-collision nobody

  if (any? balls in-radius 0.2) or (any? balls in-cone speed-tshd 185) [
    set target-approachability? true
    set p-collision [patch-here] of $me
    stop
  ]

  if ([dis-move] of $ball = 0) [
    set p-collision [patch-here] of $ball
    if (member? [patch-here] of $ball role-box-check) [
      set xtarget ([xcor] of $ball)
      set ytarget ([ycor] of $ball)
      set target-approachability? true
    ]
    stop
  ]

  let hypotenuse 0
  let opposite 0
  let adjucent 0
  let #link-23-24 nobody
  let $angle1 0
  let $angle2 0
  let travel-t 2
  let t 1

  ask $ball [
    create-link-to turtle 24
    set #link-23-24 link 23 24
    while [( t - travel-t < 1 ) and (t < 50) and (can-move? (t * dis-move)) and (member? patch-ahead (t * dis-move) pitch-patches)] [
      set xtarget (xcor  + ((t * dis-move) * dx))
      set ytarget (ycor + ((t * dis-move) * dy))
      if (xtarget = [xcor] of $me and ytarget = [ycor] of $me) [ set xtarget (xtarget + 0.0001) ]
      ask turtle 24 [ setxy xtarget ytarget ]

      ask $me [ ;; using the Pythagoras' theorem to calculate the colliding location, if any, between the ball and the current player
        set $angle1 abs [subtract-headings link-heading [towards $me] of end1] of #link-23-24
        set $angle2 abs [subtract-headings (180 + link-heading) [towards $me] of end2] of #link-23-24
        ifelse ($angle1 < 90 and $angle2 < 90) [
          set hypotenuse (distance [end1] of #link-23-24)
          set opposite (hypotenuse * sin $angle1)
          set adjucent (hypotenuse * cos $angle1)
          set xtarget ([xcor] of $ball + (adjucent * [dx] of $ball))
          set ytarget ([ycor] of $ball + (adjucent * [dy] of $ball))
        ] [
          set opposite distancexy xtarget ytarget
        ]
        set travel-t floor (opposite / speed-tshd)
      ]
      set t (t + 1)
    ]

    if (t - travel-t >= 1) and (member? patch xtarget ytarget role-box-check) [ set target-approachability? true ]

    if (not can-move? (t * dis-move) or not member? patch-ahead (t * dis-move) pitch-patches) and
       (member? patch xtarget ytarget role-box-check) [ set target-approachability? true ]

    set p-collision patch xtarget ytarget
  ]
end

to goalie-action ;; to instruct goalies when they neaither possess the bal nor can approach it
  let dis-ball distance $ball
  let ptarget nobody
  let p-clos-to-ball nobody

  if (dis-ball >= 120) [ ;; goalies' version of random walk
    ifelse (($cur-owner != nobody) and (member? $cur-owner teammates)) [ set action 7 set act-7 (act-7 + 1) ][ set action 1 set act-1 (act-1 + 1) ]
    ifelse (random 2 = 0) [ set heading (heading + random-float dir-tshd) ] [ set heading (heading - random-float dir-tshd) ]
    set dis-move (0.2 * speed-tshd)
  ]

  if (dis-ball >= 60) and (dis-ball < 120) [
    ifelse (($cur-owner != nobody) and (member? $cur-owner teammates)) [ set action 7 set act-7 (act-7 + 1) ][ set action 2 set act-2 (act-2 + 1) ]
    ;; to assign the center of gaolies' role-boxes as their target
    set ptarget patch (mean [pxcor] of role-box-check) (mean [pycor] of role-box-check)
    ;; to make sure the goalie is not already on the target location
    ifelse (patch-here != ptarget) [ face ptarget set dis-move (0.3 * speed-tshd) ] [ face $ball set dis-move 0 ]
  ]

  if (distance $ball < 60) [
    ifelse (($cur-owner != nobody) and (member? $cur-owner teammates)) [ set action 7 set act-7 (act-7 + 1) ][ set action 3 set act-3 (act-3 + 1) ]
    let goal-center patch ([pxcor] of p-our-goal) 0  ;; to make the goalies aware of the center of their own gaol
    ifelse ($me = player 1) [
      set p-clos-to-ball min-one-of (patches with [pxcor = -105 and pycor > -7 and pycor < 7]) [distance $ball]
    ] [
      set p-clos-to-ball min-one-of (patches with [pxcor = 105 and pycor > -7 and pycor < 7]) [distance $ball]
    ]

    ifelse (([pycor] of $ball < -10) or ([pycor] of $ball > 10)) [
      ask turtle 24 [ move-to p-clos-to-ball facexy 0 0 set ptarget patch-ahead ((distance $ball) / 10) ]
    ][
      ask turtle 24 [ move-to goal-center facexy 0 0 set ptarget patch-ahead ((distance $ball) / 10) ]
    ]

    face ptarget
    ;; to stop the goalie, if it is already on the target location
    if (patch-here = ptarget) [ face $ball set dis-move 0 ]
    ifelse (speed-tshd <= distance ptarget) [ set dis-move speed-tshd ] [ set dis-move (distance ptarget) ]
  ]
end

to watching-opponent ;; to watch the direct/targeted opponent
  set action 2
  set act-2 (act-2 + 1)
  let ptarget nobody
  let teamee nobody

  let trg1 (player fir-opp)
  let trg2 (player fir-opp)
  ;; to assign the first marked opponent again for the second marked, opponent, if the current player has only marked one opponent
  if (sec-opp != "Null") [ set trg2 player sec-opp ]

  ask trg1 [ set ptarget patch-at-heading-and-distance (towards p-our-goal) 6 set teamee teammates in-radius 10 ] ;; 6 patches away from the marked opponent toward our-goal
  if (not member? ptarget role-box-check) or (any? teamee) [ ;; to check if the first marked opponent is inside the current player's role-box
    ask trg2 [ set ptarget patch-at-heading-and-distance (towards p-our-goal) 6 set teamee teammates in-radius 10 ]
  ]

  if (not member? ptarget role-box-check) or (any? teamee) [ ;; to check if the fourth marked opponent is inside the current player's role-box
    ;; if none of the marked oppennets are inside the role-box, move towards the closest edge the role-box to your team's goal
    set ptarget one-of (min-n-of 20 role-box-check [distance p-our-goal])
  ]

  face ptarget
  set dis-move (0.3 * speed-tshd)
end

to random-walk ;; to walk randomly, which  give players a degree of freedom
  set action 1
  set act-1 (act-1 + 1)

  let m-pycor (mean [pycor] of role-box-check)
  let close-patches min-n-of 5 (role-box-check with [pycor = m-pycor]) [distance $ball]
  let l-pycor (min [pycor] of role-box-check)
  let h-pycor (max [pycor] of role-box-check)

  face one-of role-box-check with [pxcor = [pxcor] of one-of close-patches and pycor >= (l-pycor + 6) and pycor <= (h-pycor - 6)]

;  ifelse (random 2 = 0) [ set heading (heading + random dir-tshd) ] [ set heading (heading - random dir-tshd) ]

  set dis-move random-float (0.3 * speed-tshd)

  if (not member? patch-ahead dis-move role-box-check) [ ;; to check the first-order in this random walk
    face role-center ;; if next step is out of the role box area, turn toward the center of the role box
  ]
end

to get-the-ball ;; to guide players to get the ball
  set action 3
  set act-3 (act-3 + 1)
  let opp-corners (patch-set (patch ([pxcor] of p-opp-goal) -60)  (patch ([pxcor] of p-opp-goal) 60)) ;; finding the oppoenents corner points
;  let role-box-up-front-corner min-one-of (role-box-check with-max [pycor]) [distance p-opp-goal] ;; the top corner of player's role-box
;  let role-box-dn-front-corner min-one-of (role-box-check with-min [pycor]) [distance p-opp-goal] ;; the down corner of player's role-box
;  let role-box-corners (patch-set role-box-up-front-corner role-box-dn-front-corner) ;; ;; both corners of player's role-box

  if (any? balls in-cone speed-tshd 185) or (any? balls in-radius 0.1) [ ;; get the ball, if it is very close
    if ($cur-owner != nobody) and ($cur-owner != $me) [ ask $cur-owner [ set $last-owner self ] ]
    set $cur-owner self
    ask $ball [ move-to $me set dis-move 0 ]
    set dest-target one-of patches with [pxcor = [pxcor] of p-opp-goal]
;    set dest-target min-one-of opp-corners [distance self] ;; you can change this to make players always go towards the corner of the pitch
;    set dest-target one-of role-box-corners
    stop
  ]

  if (member? role goalies) and (distance $ball < 10) [ ;; making the golies able to dive 5 patches to get the ball
    if (any? balls in-radius 1) [ set $cur-owner self ask $ball [ move-to $me set dis-move 0 ] ]
    let pdive (min-n-of 6 patches with [pxcor = [pxcor] of $me] [distance $me])
    let ball-patch-ahead [patch-here] of $ball
    let dis [dis-move] of $ball
    while [dis > -0.4] [
      if (member? ball-patch-ahead pdive) [
        face ball-patch-ahead
        set dis-move (distance ball-patch-ahead)
        ask $ball [ set dis-move (distance ball-patch-ahead) ]
        set dest-target min-one-of opp-corners [distance self] ;; you can change this to make players always go towards the corner of the pitch
                                                                  ; set dest-target one-of role-box-corners
        stop
      ]
      set ball-patch-ahead [patch-ahead dis] of $ball
      set dis (dis - 0.5)
    ]
  ]

  let dis-to-location distancexy xtarget ytarget
  facexy xtarget ytarget

  ifelse (dis-to-location < speed-tshd) [ ;; to slow down in the last step if the target location is close
    set dis-move dis-to-location
    ask $ball [
      let xtarget-i (xcor + (dis-move * dx))
      let ytarget-i (ycor + (dis-move * dy))
      if (xtarget-i = [xcor] of $me and ytarget-i = [ycor] of $me) [ set dis-move distancexy xtarget ytarget stop ]
      if (xtarget-i = xcor) and (ytarget-i = ycor) [ set xtarget-i (xtarget-i + 0.0001) ]
      ask turtle 24 [ setxy xtarget-i ytarget-i ]
      create-link-to turtle 24
      let #link-23-24 link 23 24
      let $angle1 abs [subtract-headings link-heading [towards $me] of end1] of #link-23-24
      let $angle2 abs [subtract-headings (180 + link-heading) [towards $me] of end2] of #link-23-24
      if ($angle1 < 1 and $angle2 < 1) [ set dis-move distancexy xtarget ytarget ]
    ]
  ] [
    set dis-move speed-tshd
  ]
end

to positioning ;; to find a good position to open up the game when one of teammates possesses the ball
  set action 7
  set act-7 (act-7 + 1)
  let target nobody
  let close-opp min-one-of (min-n-of 2 opp-players [distance self]) [distance p-opp-goal]
  let close-op-corner min-one-of (patch-set oppLCor oppRCor) [distance self]
  let last-def one-of opp-players with-min [distance p-opp-goal] ;; to find the last current defence in the opponent team
  let px-target [pxcor] of (min-one-of role-box-check [distance p-opp-goal])
;  let py-target [pycor] of one-of (max-n-of 10 role-box-check [distance $ball])
  let py-target [pycor] of (max-one-of role-box-check [distance patch 0 0])
;
;  ifelse (distance p-opp-goal >= 30) [ ;; players concentrate on the opponent goal more, when they get closer to it
;    facexy px-target py-target
;  ][
;    face p-opp-goal
;  ]

  ifelse (distance p-opp-goal >= 30) [ ;; players concentrate on the opponent goal more, when they get closer to it
    face close-op-corner
  ][
    face p-opp-goal
  ]

  set dis-move (0.3 * speed-tshd)

  ifelse (team = "team-A") [ ;; to employ the offside rule
    if (xcor > ([xcor] of last-def)) [
      let x-target [xcor] of last-def
      let y-target mean [pycor] of role-box-check
      facexy x-target y-target
    ]
  ][
    if (xcor < ([xcor] of last-def)) [
      let x-target [xcor] of last-def
      let y-target mean [pycor] of role-box-check
      facexy x-target y-target
    ]
  ]
  if (not member? patch-ahead dis-move role-box-check) [ face role-center ]
end

to ball-owner-action
  ask $me [
    let left-tshd ([patch-here] of turtle 25) ;; turtle 25 was located at the patch that the current player can at max move to at its left side
    let rght-tshd ([patch-here] of turtle 26) ;; turtle 26 was located at the patch that the current player can at max move to at its right side
    let free-teammates nobody
    foreach sort teammates [ p ->
      face p
      if (not any? opponents in-cone (distance p) 20) [ set free-teammates (turtle-set free-teammates p) ]
    ]
    face dest-target

    let shoot-destin one-of (patches with [(pxcor = [pxcor] of p-opp-goal) and (pycor > -15 and pycor < 15)])
    let close-opp opponents in-cone 4 200
    let num-close-opp count close-opp
    let energy% (energy / 100)
    let shoot-speed ((shooting / 100) * energy% * 6) ;; 6 is the max possible speed of the ball, in case of shooting, which is 3 times more than players' max speed
    let pass-speed (shoot-speed / 2) ;; 3 is the max possible speed of the ball, in case of passing, which is 1.5 times more than players' max speed



    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; if the current player is a goalie ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (member? role goalies) [
      if (any? opponents in-cone 20 360) [ ;; kick the ball away
        let target nobody
        ifelse (pycor >= 0) [
          ifelse (pxcor > 0) [
            set target one-of patches with [(pxcor = ([pxcor] of min-one-of opponents [distance self] + 5)) and (pycor > 40)] ;; the closest opponent
          ][
            set target one-of patches with [(pxcor = ([pxcor] of min-one-of opponents [distance self] - 5)) and (pycor > 40)] ;; the closest opponent
          ]
        ][
          ifelse (pxcor > 0) [
            set target one-of patches with [(pxcor = ([pxcor] of min-one-of opponents [distance self] + 5)) and (pycor < -40)] ;; the closest opponent
          ][
            set target one-of patches with [(pxcor = ([pxcor] of min-one-of opponents [distance self] - 5)) and (pycor < -40)] ;; the closest opponent
          ]
        ]
        ask $ball [ face target set dis-move shoot-speed ]
        face role-center
        set $cur-owner nobody
        set $last-owner self
        set action 6
        set act-6 (act-6 + 1)
        stop ;; this one
      ]

      if (not any? opponents in-cone 20 360) and (any? opponents in-cone 40 360) [ ;; pass the ball to a random teammate
        ask $ball [
          ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
          set dis-move pass-speed
        ]
        face role-center
        set $cur-owner nobody
        set $last-owner self
        set action 5
        set act-5 (act-5 + 1)
        stop ;; this one
      ]

      ;; carry the ball forward, if the next step wiil still be inside the role-box, otherwise pass it to a random teammate
      ifelse (not any? opponents in-cone 40 360) and (member? patch-ahead speed-tshd role-box-check) [
        set dis-move (0.5 * speed-tshd)
        ask $ball [ set heading [heading] of $me set dis-move [dis-move] of $me ]
        set action 4
        set act-4 (act-4 + 1)
        stop ;; this one
      ][
        ask $ball [;; pass the ball to a random teammate
          ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
          set dis-move pass-speed
        ]
        face role-center
        set $cur-owner nobody
        set $last-owner self
        set action 5
        set act-5 (act-5 + 1)
        stop ;; this one
      ]
    ]
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; if the current player is a defender ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (member? role defenders) [
      if (any? opponents in-radius 6) [
        ifelse (distance p-our-goal < 20) [ ;; kick the ball to a random location, if its close to the team's goal
          ask $ball [ face dest-target set dis-move shoot-speed ]
          set $last-owner self
          set $cur-owner nobody
          set action 6
          set act-6 (act-6 + 1)
        ] [
          ask $ball [;; pass the ball to a random teammate
            ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
            set dis-move pass-speed
          ]
          set $last-owner self
          set $cur-owner nobody
          set action 5
          set act-5 (act-5 + 1)
        ]
        stop ;; this one
      ]
      ;; carry the ball forward, if the next step wiil still be inside the role-box, otherwise pass it to a random teammate
      if (not any? opponents in-radius 6) [
        ifelse (member? patch-ahead speed-tshd role-box-check) [
          set dis-move speed-tshd
          ask $ball [ set heading [heading] of $me set dis-move [dis-move] of $me ]
          set action 4
          set act-4 (act-4 + 1)
        ] [
          ask $ball [;; pass the ball to a random teammate
            ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
            set dis-move pass-speed
          ]
          set $last-owner self
          set $cur-owner nobody
          set action 5
          set act-5 (act-5 + 1)
        ]
        stop ;; this one
      ]
    ]
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; if the current player is a midfielder ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (member? role midfielders) [
      if (any? close-opp) [
        ;; pass the ball to a random teammate
        if ((dir-tshd <= 30) and (speed-tshd <= 1) or (not member? left-tshd role-box-check) or (not member? rght-tshd role-box-check) or (num-close-opp > 1)) [
          ask $ball [;; pass the ball to a random teammate
            ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
            set dis-move pass-speed
          ]
          set $last-owner self
          set $cur-owner nobody
          set action 5
          set act-5 (act-5 + 1)
        ]
        ;; dribble the ball
        if ((dir-tshd > 30) or (speed-tshd > 1) and (member? left-tshd role-box-check) and (member? rght-tshd role-box-check) and (num-close-opp <= 1)) [
          let dis1 0
          let dis2 0
          ask min-one-of close-opp [distance $me] [ set dis1 distance turtle 25 set dis2 distance turtle 26 ]
          ifelse (dis1 <= dis2) [ ;; to decide to go left or right
            face turtle 26
            set dis-move speed-tshd
            ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
          ][
            face turtle 25
            set dis-move speed-tshd
            ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
          ]
          set action 4
          set act-4 (act-4 + 1)
        ]
        stop ;; this one
      ]
      ;; carry the ball forward, if the next step wiil still be inside the role-box, otherwise pass it to a random teammate
      if (not any? close-opp) [
        ifelse (member? patch-ahead speed-tshd role-box-check) [
          set dis-move speed-tshd
          ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
          set action 4
          set act-4 (act-4 + 1)
        ] [
          let opps-in-cone count (opponents in-cone (distance p-opp-goal) 45)
          ifelse (opps-in-cone <= 1) [
            ask $ball [ face p-opp-goal set dis-move (shoot-speed) ]
            set $last-owner self
            set $cur-owner nobody
            set action 6
            set act-6 (act-6 + 1)
          ][
            ask $ball [;; pass the ball to a random teammate
              ifelse (free-teammates != nobody) [ face min-one-of free-teammates [distance p-opp-goal] ] [ face one-of (max-n-of 6 teammates [distance p-opp-goal]) ]
              set dis-move pass-speed
            ]
            set $last-owner self
            set $cur-owner nobody
            set action 5
            set act-5 (act-5 + 1)
          ]
        ]
        stop ;; this one
      ]
    ]
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; if the current player is a striker ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (member? role strikers) [
      ifelse (distance p-opp-goal < 25) [ ;; shoot the ball, if close to the opponent' goal
          ask $ball [ face p-opp-goal set dis-move (shoot-speed) ]
          set $last-owner self
          set $cur-owner nobody
          set action 6
          set act-6 (act-6 + 1)
        ][ ;; otherwise ...
          if (any? close-opp) [
           ;; pass the ball to a random teammate, if can't dribble the close opponent
           if ((dir-tshd <= 30) and (speed-tshd <= 1) or (not member? left-tshd role-box-check) or (not member? rght-tshd role-box-check) or (num-close-opp > 1)) [
             ask $ball [ face one-of [neighbors] of one-of teammates set dis-move pass-speed ]
             set $last-owner self
             set $cur-owner nobody
             set action 5
             set act-5 (act-5 + 1)
           ]
           ;; dribble the close opponent, if can
           if ((dir-tshd > 30) or (speed-tshd > 1) and (member? left-tshd role-box-check) and (member? rght-tshd role-box-check) and (num-close-opp <= 1)) [
             let dis1 0
             let dis2 0
             ask min-one-of close-opp [distance $me] [ set dis1 distance turtle 25 set dis2 distance turtle 26 ]
             ifelse (dis1 <= dis2) [
               face turtle 26
               set dis-move speed-tshd
               ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
             ] [
               face turtle 25
               set dis-move speed-tshd
               ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
             ]
             set action 4
             set act-4 (act-4 + 1)
           ]
         ]
         if (not any? close-opp) [ ;; carry the ball forward, if no opponent is close
          face p-opp-goal
          set dis-move speed-tshd
          ask $ball [ set heading [heading] of $me set dis-move [speed-tshd] of $me ]
          set action 4
          set act-4 (act-4 + 1)
        ]
      ]
    ]
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ]
end

to make-decission ;; a combinaton of all factor-orders mostly the second-order influencing factors
  if (member? role goalies) [ ;; if caller is a goalie

    if (($cur-owner != nobody) and (member? $cur-owner teammates)) [ set ball-possessor "Teammates" goalie-action ]

    if (($cur-owner = nobody) or (member? $cur-owner opponents)) [
      ifelse ($cur-owner = nobody) [ set ball-possessor "Nobody" ] [ set ball-possessor "Opponents" ]

;      check-approachability
      ifelse (target-approachability? = true) [ get-the-ball ] [ goalie-action ]
    ]
  ]

  if (not member? role goalies) [ ;; if caller is not a goalie
    if ($cur-owner = nobody) [
      set ball-possessor "Nobody"
;      check-approachability
      if (target-approachability? = true) [ get-the-ball ]
      if (target-approachability? = false) [
        ifelse (distance $ball < 80) [ watching-opponent ] [ random-walk ]
      ]
    ]

    if ($cur-owner != nobody) and ($cur-owner != $me) [
      if (member? $cur-owner opponents) [
        set ball-possessor "Opponents"
;        check-approachability
        if (target-approachability? = true) [ get-the-ball ]
        if (target-approachability? = false) [
          ifelse (distance $ball < 100) [ watching-opponent ] [ random-walk ]
        ]
      ]
      if (member? $cur-owner teammates) [
        set ball-possessor "Teammates"
        ifelse (distance $ball < 80) [ positioning ] [ random-walk ]
      ]
    ]
  ]

  if ($cur-owner = $me) [
    set ball-possessor "Me"
    ball-owner-action
  ]
end

to go
  ;; these two below lines are only for when the game is set to run for a long time
  if (ticks = 9000) [ export-data ask players [ set exp-data [] ] ]
  if (ticks = 18000) [ set NGame (NGame + 1) export-data ask players [ set exp-data [] ] ]
  if (ticks > NTicks) [ set NGame (NGame + 1) export-data ask players [ set exp-data [] ] ]

  if (ticks > NTicks) [ ;; 1 half time
    ask players [ print (word "Player_" who "(act-1: " act-1 "; act-2: " act-2  "; act-3: " act-3  "; act-4: " act-4  "; act-5: " act-5  "; act-6: " act-6  "; act-7: " act-7 ")") ]
    print (word "team_A_scores: " team_A_scores "; team_B_scores: " team_B_scores)
    user-message " The End "
    stop
  ]

  ask players [
    random-seed new-seed
    set $me self ;; remember me during the whole decision making process
    check-zero-order
    check-first-order
    check-second-order
    check-approachability
    set p-heading heading
    make-decission ;; a combinaton of all factor-orders
    if (not member? patch-here role-box-check) [ face role-center ]
    set-exp-data
    fd dis-move
    set running (running + dis-move)
    display
  ]

  ask $ball [
    if (patch-ahead dis-move = nobody) [ while [patch-ahead dis-move = nobody] [ set dis-move (dis-move - 1) ] ] ;; to prevent the ball crossing the world

    if (not member? patch-ahead dis-move pitch-patches) [
      let step 0
      while [step <= dis-move] [
        if (member? patch-ahead step team-A-goal) [ ;; if team A recieves a goal
          set team_B_scores team_B_scores + 1
          set $last-owner nobody
          role-positioning
          let midfielder min-one-of (players with [team = "team-A"]) [distance (patch 0 0)]
          ask midfielder [ setxy 0 0 ]
          move-to midfielder
          set dis-move 0
        ]
        if (member? patch-ahead step team-B-goal) [ ;; if team A recieves a goal
          set team_A_scores team_A_scores + 1
          set $last-owner nobody
          role-positioning
          let midfielder min-one-of (players with [team = "team-B"]) [distance (patch 0 0)]
          ask midfielder [ setxy 0 0 ]
          move-to midfielder
          set dis-move 0
        ]
        set step (step + 0.5)
      ]

      if (not member? patch-ahead dis-move pitch-patches) [ set dis-move 0 ] ;; to prevent the ball crossing the pitch
    ]
    fd dis-move
    display
  ]

  calculate-actions
  if (turn-on-act-plots? = true) [ act-plots ]
  if (turn-on-traj-plots? = true) [ trajectory-plots ]

  ;   rec
  tick
end

to act-plots
  let counter 1
  repeat 7 [
    let pName (word "Act-" counter)
    set-current-plot pName

    let val 0

    let p-counter 1
    repeat 22 [
      if (counter = 1) [ set val ([Act-1] of player p-counter) ]
      if (counter = 2) [ set val ([Act-2] of player p-counter) ]
      if (counter = 3) [ set val ([Act-3] of player p-counter) ]
      if (counter = 4) [ set val ([Act-4] of player p-counter) ]
      if (counter = 5) [ set val ([Act-5] of player p-counter) ]
      if (counter = 6) [ set val ([Act-6] of player p-counter) ]
      if (counter = 7) [ set val ([Act-7] of player p-counter) ]

      if (p-counter <= 11) [set-current-plot-pen _clarify-duplicate-plot-pen-name "TeamA-pen" set-plot-pen-color Blue plotxy (p-counter) (val) ]
      if (p-counter > 11) [set-current-plot-pen _clarify-duplicate-plot-pen-name "TeamA-pen" set-plot-pen-color Red plotxy (p-counter - 0.3) (val) ]

      set p-counter (p-counter + 1)
    ]

    set counter (counter + 1)
  ]

end

to trajectory-plots
  set-current-plot "Trajectory of Player 10"
  set-current-plot-pen _clarify-duplicate-plot-pen-name "Trajectory"
  plotxy ([xcor] of player 10) ([ycor] of player 10)

  set-current-plot "Trajectory of Ball"
  set-current-plot-pen _clarify-duplicate-plot-pen-name "Trajectory"
  plotxy ([xcor] of $ball) ([ycor] of $ball)

end

to calculate-actions
  set actions (list (sum [act-1] of players) (sum [act-2] of players) (sum [act-3] of players) (sum [act-4] of players) (sum [act-5] of players) (sum [act-6] of players) (sum [act-7] of players))
end

to set-exp-data
  ask $me [
    let observations []
    set observations lput (round  ticks) observations
    set observations lput who observations
    set observations lput stumina observations
    set observations lput pace observations
    set observations lput agility observations
    set observations lput shooting observations
    set observations lput teamwork observations
    set observations lput (precision xcor 1) observations
    set observations lput (precision ycor 1) observations
    set observations lput (precision energy 2) observations
    set observations lput (precision (energy * shooting / 100) 1) observations
    set observations lput (precision (energy * teamwork / 100) 1) observations
    set observations lput (precision speed-tshd 1) observations
    set observations lput (precision dir-tshd 1) observations
    set observations lput (round  heading) observations
    set observations lput (round  p-heading) observations
    set observations lput (precision dis-move 1) observations
    set observations lput (round  running) observations
    set observations lput action observations

    ;; calculating distance, absolute and relative bearing to other players
    foreach sort players [ p ->
      ask self [
        ifelse ((distance p) = 0) [
          set observations lput 0 observations ;; disToPlayers
          set observations lput 0 observations ;; absDirToPlayers
          set observations lput 0 observations ;; relDirToPlayers
        ] [
          set observations lput (precision (distance p) 1) observations ;; disToPlayers
          set observations lput (round  (towards p)) observations ;; absDirToPlayers
          set observations lput (round  (subtract-headings (towards p) p-heading)) observations ;; relDirToPlayers
        ]
      ]
    ]

    ;; calculating distance, absolute and relative bearing to the left corner of the opponent's side
    ifelse ((distance oppLCor) = 0) [
      set observations lput 0 observations ;; disToRole-center
      set observations lput 0 observations ;; absDirToRole-center
      set observations lput 0 observations ;; relDirToRole-center
    ] [
      set observations lput (precision (distance oppLCor) 1) observations ;; disToRole-center
      set observations lput (round  (towards oppLCor)) observations ;; absDirToRole-center
      set observations lput (round  (subtract-headings (towards oppLCor) p-heading)) observations ;; relDirToRole-center
    ]

    ;; calculating distance, absolute and relative bearing to the right corner of the opponent's side
    ifelse ((distance oppRCor) = 0) [
      set observations lput 0 observations ;; disToRole-center
      set observations lput 0 observations ;; absDirToRole-center
      set observations lput 0 observations ;; relDirToRole-center
    ] [
      set observations lput (precision (distance oppRCor) 1) observations ;; disToRole-center
      set observations lput (round  (towards oppRCor)) observations ;; absDirToRole-center
      set observations lput (round  (subtract-headings (towards oppRCor) p-heading)) observations ;; relDirToRole-center
    ]

    ask oppLCor [ set observations lput (precision (distance $ball) 1) observations ] ;; Ball's distance to the left corner of the opponent's side
    ask oppRCor [ set observations lput (precision (distance $ball) 1) observations ] ;; Ball's the left corner of the opponent's side

    ;; calculating distance, absolute and relative bearing to players' role-centers
    ifelse ((distance role-center) = 0) [
      set observations lput 0 observations ;; disToRole-center
      set observations lput 0 observations ;; absDirToRole-center
      set observations lput 0 observations ;; relDirToRole-center
    ] [
      set observations lput (precision (distance role-center) 1) observations ;; disToRole-center
      set observations lput (round  (towards role-center)) observations ;; absDirToRole-center
      set observations lput (round  (subtract-headings (towards role-center) p-heading)) observations ;; relDirToRole-center
    ]

    ;; calculating distance, absolute and relative bearing to team_A_goal
    let goalA patch (mean [pxcor] of team-A-goal) (mean [pycor] of team-A-goal)
    ifelse ((distance goalA) = 0) [
      set observations lput 0 observations ;; disToOurGoalCenter
      set observations lput 0 observations ;; absDirToOurGoalCenter
      set observations lput 0 observations ;; relDirToOurGoalCenter
    ] [
      set observations lput (precision (distance goalA) 1) observations ;; disToOurGoalCenter
      set observations lput (round  (towards goalA)) observations ;; absDirToOurGoalCenter
      set observations lput (round  (subtract-headings (towards goalA) p-heading)) observations ;; relDirToOurGoalCenter
    ]

    ;; calculating distance, absolute and relative bearing to team_B_goal
    let goalB patch (mean [pxcor] of team-B-goal) (mean [pycor] of team-B-goal)
    ifelse ((distance goalB) = 0) [
      set observations lput 0 observations ;; disToOurGoalCenter
      set observations lput 0 observations ;; absDirToOurGoalCenter
      set observations lput 0 observations ;; relDirToOurGoalCenter
    ] [
      set observations lput (precision (distance goalB) 1) observations ;; disToOurGoalCenter
      set observations lput (round  (towards goalB)) observations ;; absDirToOurGoalCenter
      set observations lput (round  (subtract-headings (towards goalB) p-heading)) observations ;; relDirToOurGoalCenter
    ]

    ;; calculating distance, absolute and relative bearing to players' own goal
    ifelse ((distance p-our-goal) = 0) [
      set observations lput 0 observations ;; disToOurGoalCenter
      set observations lput 0 observations ;; absDirToOurGoalCenter
      set observations lput 0 observations ;; relDirToOurGoalCenter
    ] [
      set observations lput (precision (distance p-our-goal) 1) observations ;; disToOurGoalCenter
      set observations lput (round  (towards p-our-goal)) observations ;; absDirToOurGoalCenter
      set observations lput (round  (subtract-headings (towards p-our-goal) p-heading)) observations ;; relDirToOurGoalCenter
    ]

    ;; calculating distance, absolute and relative bearing to players' opponent goal (note: this is a point inside the opponent's goal that has been targeted by the current player)
    ifelse ((distance p-opp-goal) = 0) [
      set observations lput 0 observations ;; disToOpprGoalCenter
      set observations lput 0 observations ;; absDirToOpprGoalCenter
      set observations lput 0 observations ;; relDirToOpprGoalCenter
    ] [
      set observations lput (precision (distance p-opp-goal) 1) observations ;; disToOpprGoalCenter
      set observations lput (round  (towards p-opp-goal)) observations ;; absDirToOpprGoalCenter
      set observations lput (round  (subtract-headings (towards p-opp-goal) p-heading)) observations ;; relDirToOpprGoalCenter
    ]

    ;; calculating distance, absolute and relative bearing to the ball
    ifelse ((distance $ball) < 0.5) or ($cur-owner = self) [
      set observations lput 0 observations ;; disToBall
      set observations lput 0 observations ;; absDirToBall
      set observations lput 0 observations ;; relDirToBall
    ] [
      set observations lput (precision (distance $ball) 1) observations ;; disToBall
      set observations lput (round  (towards $ball)) observations ;; absDirToPlayers
      set observations lput (round  (subtract-headings (towards $ball) p-heading)) observations ;; relDirToBall
    ]

    ask role-center [
      ifelse (distance p-collision = 0) [
        set observations lput 0 observations ;; disFromRole-CenterToCollisionPoint
        set observations lput 0 observations ;; absDirFromRole-CenterToCollisionPoint
      ] [
        set observations lput (precision (distance p-collision) 1) observations ;; disFromRole-CenterToCollisionPoint
        set observations lput (round  (towards p-collision)) observations ;; absDirFromRole-CenterToCollisionPoint
      ]
    ]

    ifelse ($cur-owner = self) or (distance p-collision < 0.2) [
      set observations lput 0 observations ;; disToCollisionPoint
      set observations lput 0 observations ;; absDirToCollisionPoint
      set observations lput 0 observations ;; relDirToCollisionPoint
    ] [
      set observations lput (precision (distance p-collision) 1) observations ;; disToCollisionPoint
      set observations lput (round  (towards p-collision)) observations ;; absDirToCollisionPoint
      set observations lput (round  (subtract-headings (towards p-collision) p-heading)) observations ;; relDirToCollisionPoint
    ]

    set observations lput target-approachability? observations ;; ballApprochability

    ;; Ball's attributes
    set observations lput (precision ([xcor] of $ball) 1) observations ;; Ball's X Coordinate
    set observations lput (precision ([ycor] of $ball) 1) observations ;; Ball's Y Coordinate
    set observations lput (round ([heading] of $ball)) observations ;; Ball's Heading
    set observations lput (precision ([dis-move] of $ball) 1) observations ;; Ball's Speed
    set observations lput (ball-possessor) observations ;; Ball_Possessor
    ifelse ($cur-owner = nobody) [ set observations lput 0 observations ] [ set observations lput ([who] of $cur-owner) observations ] ;; Ball'sCurrentPossessor
    ifelse ($last-owner = nobody) [ set observations lput 0 observations ] [ set observations lput ([who] of $last-owner) observations ] ;; Ball'sPreviousPossessor

    set exp-data lput observations exp-data
  ]
end

to export-data
  set-current-directory user-directory ;; to allow users to choose an existing directory on the system

  let header (list "T" "PId" "ST" "P" "A" "Sh" "TW" "PX" "PY" "E" "SD" "PD" "TS" "TT" "PH" "PPH" "PS" "TD" "EA"
    "DP1" "ABP1" "RBP1" "DP2" "ABP2" "RBP2" "DP3" "ABP3" "RBP3" "DP4" "ABP4" "RBP4" "DP5" "ABP5" "RBP5" "DP6" "ABP6" "RBP6"
    "DP7" "ABP7" "RBP7" "DP8" "ABP8" "RBP8" "DP9" "ABP9" "RBP9" "DP10" "ABP10" "RBP10" "DP11" "ABP11" "RBP11" "DP12" "ABP12" "RBP12"
    "DP13" "ABP13" "RBP13" "DP14" "ABP14" "RBP14" "DP15" "ABP15" "RBP15" "DP16" "ABP16" "RBP16" "DP17" "ABP17" "RBP17" "DP18" "ABP18" "RBP18"
    "DP19" "ABP19" "RBP19" "DP20" "ABP20" "RBP20" "DP21" "ABP21" "RBP21" "DP22" "ABP22" "RBP22"
    "DOLC" "ABOLC" "RBOLC" "DORC" "ABORC" "RBORC" "BDOLC" "BDORC"
    "DR" "ABR" "RBR" "DGA" "ABGA" "RBGA" "DGB" "ABGB" "RBGB" "DGOw" "ABGOw" "RBGOw" "DGOp" "ABGOp" "RBGOP"
    "DB" "ABB" "RBB" "DCPFR" "ABCPFR" "DCP" "ABCP" "RBCP" "BAp" "XB" "YB" "BH" "BS" "BP" "BPId" "BPPId")


  ask players [
    set exp-data fput header exp-data
    csv:to-file (word "Game(" a-formation "_" b-formation ")_Player_" who "_" NGame ".csv") exp-data
  ]
end

to rec
  file-open (word "Game(" a-formation "_" b-formation ")_N_.csv")
  ask players [
    file-write (word (precision ticks 1) "/" who "/" (precision xcor 2) "/" (precision ycor 2) "/" (precision heading 2) "/"
      (precision energy 4) "/" (precision speed-tshd 3) "/" (precision dir-tshd 3) "/" (precision dis-move 3) "/" (precision running 3) "/" action)

;    if (ticks <= 15000) [ file-open (word a-formation "_" b-formation "_" who "_1" ".csv") ]
;    if (ticks > 15000) [ file-open (word a-formation "_" b-formation "_" who "_2" ".csv") ]
;    file-write (word (precision ticks 1) "/" who "/" (precision xcor 2) "/" (precision ycor 2) "/" (precision heading 2) "/"
;      (precision energy 4) "/" (precision speed-tshd 3) "/" (precision dir-tshd 3) "/" (precision dis-move 3) "/" (precision running 3) "/" action "/" pDirList "/" pDisList)
;
;    file-write ","
;    file-close
;    show (word " Energy:" (precision energy 2) " Speed-tshd:" (precision speed-tshd 2) " dis-move:" (precision dis-move 1) " heading:" (precision heading 1))
  ]

  ask $ball [
;    file-open (word a-formation "_" b-formation "_Ball" ".csv")
    file-write (word (precision ticks 1) "/" who "/" (precision xcor 2) "/" (precision ycor 2) "/" (precision heading 2) "/" (precision dis-move 3) "/" $cur-owner "/" $last-owner)
;    file-write ","
;    file-close
  ]
  file-write ","
  file-close
end

to-report _clarify-duplicate-plot-pen-name [ name ]
  let name-map [["X-axis" "X-axis"] ["X-Axis" "X-Axis_1"]]
  let replacement filter [ rename -> first rename = name] name-map
  let reported-name name
  if not empty? replacement [
    set reported-name item 1 replacement
  ]
  report reported-name
end

;to profile
;  profiler:start
;  repeat 100 [ go ]
;  profiler:stop
;  print profiler:report
;  profiler:reset
;end

;; keep this in mind that the number of extra decisions in the total count of decisions is because of the "get the ball" action (you can reduce it from that).
;; the differnce between this number and the number of passess + shoots indicates how many times each player has lost the ball while deciding to carry it.
@#$#@#$#@
GRAPHICS-WINDOW
155
10
1268
654
-1
-1
5.0
1
15
1
1
1
0
0
0
1
-110
110
-63
63
1
1
1
ticks
10.0

BUTTON
5
190
65
223
Setup
setup
NIL
1
T
OBSERVER
NIL
1
NIL
NIL
1

BUTTON
80
190
140
223
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

BUTTON
5
350
65
383
Show
show-role-boxes
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
80
350
140
383
Clear
ask pitch-patches [ set pcolor 55 ]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
5
250
140
295
a-role-box
a-role-box
"none" "goalie" "lCB" "CB" "rCB" "lDM" "rDM" "lWB" "rWB" "OM" "lF" "rF" "lB" "rB" "lFB" "rFB" "CDM" "lSM" "rSM" "lM" "CM" "rM" "CF" "lW" "rW"
9

CHOOSER
5
300
140
345
b-role-box
b-role-box
"none" "goalie" "lCB" "CB" "rCB" "lDM" "rDM" "lWB" "rWB" "OM" "lF" "rF" "lB" "rB" "lFB" "rFB" "CDM" "lSM" "rSM" "lM" "CM" "rM" "CF" "lW" "rW"
0

CHOOSER
5
90
140
135
a-formation
a-formation
"3-5-2" "4-4-2" "4-3-3"
0

CHOOSER
5
140
140
185
b-formation
b-formation
"3-5-2" "4-4-2" "4-3-3"
0

MONITOR
5
430
70
479
Barcelona
team_A_scores
0
1
12

MONITOR
80
430
140
479
Madrid
team_B_scores
0
1
12

TEXTBOX
55
405
100
423
Scores
15
0.0
1

PLOT
1277
670
1577
830
Trajectory of Player 10
X
Y
-110.0
110.0
-60.0
60.0
true
false
"" ""
PENS
"X-Axis_1" 1.0 0 -1184463 true "plot-pen-reset\nplotxy -110 0\nplotxy 110 0" ""
"Y-axis" 1.0 0 -1184463 true "plot-pen-reset\nplotxy 0 -60\nplotxy 0 60" ""
"Trajectory" 1.0 0 -13345367 true "" "; plotxy ([xcor] of player 10) ([ycor] of player 10)"

SLIDER
5
50
140
83
NTicks
NTicks
0
27000
2400.0
100
1
NIL
HORIZONTAL

PLOT
1582
505
1882
665
Exacuted actions by all Players
Ticks
Frequency (%)
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"Act-1" 1.0 0 -5207188 true "" "let action%1 ((item 0 actions) / (sum actions)) * 100\nplot action%1"
"Act-2" 1.0 0 -8630108 true "" "let action%2 ((item 1 actions) / (sum actions)) * 100\nplot action%2"
"Act-3" 1.0 0 -13791810 true "" "let action%3 ((item 2 actions) / (sum actions)) * 100\nplot action%3"
"Act-4" 1.0 0 -13840069 true "" "let action%4 ((item 3 actions) / (sum actions)) * 100\nplot action%4"
"Act-5" 1.0 0 -2674135 true "" "let action%5 ((item 4 actions) / (sum actions)) * 100\nplot action%5"
"Act-6" 1.0 0 -14737633 true "" "let action%6 ((item 5 actions) / (sum actions)) * 100\nplot action%6"
"Act-7" 1.0 0 -7500403 true "" "let action%7 ((item 6 actions) / (sum actions)) * 100\nplot action%7"

PLOT
1277
10
1577
170
Act-1
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ";plotxy 1 ([act-1] of player 1)\n;plotxy 2 ([act-1] of player 2)\n;plotxy 3 ([act-1] of player 3)\n;plotxy 4 ([act-1] of player 4)\n;plotxy 5 ([act-1] of player 5)\n;plotxy 6 ([act-1] of player 6)\n;plotxy 7 ([act-1] of player 7)\n;plotxy 8 ([act-1] of player 8)\n;plotxy 9 ([act-1] of player 9)\n;plotxy 10 ([act-1] of player 10)\n;plotxy 11 ([act-1] of player 11)"
"pen-1" 1.0 0 -16777216 false ";plot-pen-reset\n;plotxy 11.7 0\n;plotxy 11.7 2000" ""
"TeamB-pen" 0.3 1 -2674135 true "" ";plotxy 11.7 ([act-1] of player 12)\n;plotxy 12.7 ([act-1] of player 13)\n;plotxy 13.7 ([act-1] of player 14)\n;plotxy 14.7 ([act-1] of player 15)\n;plotxy 15.7 ([act-1] of player 16)\n;plotxy 16.7 ([act-1] of player 17)\n;plotxy 17.7 ([act-1] of player 18)\n;plotxy 18.7 ([act-1] of player 19)\n;plotxy 19.7 ([act-1] of player 20)\n;plotxy 20.7 ([act-1] of player 21)\n;plotxy 21.7 ([act-1] of player 22)"

PLOT
1582
10
1882
170
Act-2
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

PLOT
1582
340
1882
500
Act-6
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

PLOT
1277
505
1577
665
Act-7
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

PLOT
1277
175
1577
335
Act-3
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

PLOT
1277
340
1577
500
Act-5
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

PLOT
1582
175
1882
335
Act-4
Player
Frequency
1.0
22.0
0.0
1.0
true
false
"" ""
PENS
"TeamA-pen" 0.3 1 -13345367 true "" ""
"pen-1" 1.0 0 -1 true "" ""
"TeamB-pen" 0.3 1 -2674135 true "" ""

SWITCH
5
490
140
523
turn-on-act-plots?
turn-on-act-plots?
0
1
-1000

PLOT
1582
670
1882
830
Trajectory of Ball
X
Y
-110.0
110.0
-60.0
60.0
true
false
"" ""
PENS
"X-axis" 1.0 0 -1184463 true "plot-pen-reset\nplotxy -110 0\nplotxy 110 0" ""
"Y-axis" 1.0 0 -1184463 true "plot-pen-reset\nplotxy 0 -60\nplotxy 0 60" ""
"Trajectory" 1.0 0 -16777216 true "" ""

SWITCH
5
530
140
563
turn-on-traj-plots?
turn-on-traj-plots?
0
1
-1000

BUTTON
5
570
140
605
Export observations
export-data
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
5
10
140
43
NGame
NGame
1
100
1.0
1
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

ball
false
0
Circle -7500403 true true 16 16 268
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269
Polygon -1 true false 150 29 116 34 88 47 135 125 145 123 150 122
Polygon -1 true false 213 47 240 69 254 89 174 137 167 124
Polygon -1 true false 179 148 177 165 252 212 264 187 268 166 270 152
Polygon -1 true false 162 177 150 179 151 269 171 268 187 265 211 254 165 173
Polygon -1 true false 124 167 130 174 137 176 88 254 72 241 59 229 47 214
Polygon -1 true false 125 134 122 139 123 152 30 151 31 126 36 111 46 86

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

person_l
false
0
Circle -7500403 true true 117 28 66
Polygon -7500403 true true 120 105 120 180 105 285 90 300 120 300 150 195 190 284 180 300 210 300 180 180 180 105
Rectangle -7500403 true true 135 90 165 105
Polygon -7500403 true true 142 105 103 166 66 166 66 144 89 144 112 105

person_r
false
0
Circle -7500403 true true 117 29 66
Polygon -7500403 true true 180 105 180 180 195 285 210 300 180 300 150 195 110 284 120 300 90 300 120 180 120 105
Rectangle -7500403 true true 135 90 165 105
Polygon -7500403 true true 158 105 197 166 234 166 234 144 211 144 188 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

test
1.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 0 150 150
@#$#@#$#@
1
@#$#@#$#@
